<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

/**
 * Crypt_RSA allows to do following operations:
 *     - key pair generation
 *     - encryption and decryption
 *     - signing and sign validation
 *
 * PHP versions 4 and 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   Encryption
 * @package    Crypt_RSA
 * @author     Alexander Valyalkin <valyala@gmail.com>
 * @copyright  2005 Alexander Valyalkin
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    1.0.0
 * @link       http://pear.php.net/package/Crypt_RSA
 */

require_once(dirname(__FILE__) . '/RSAMathLoader.php'); // loader for math wrappers
require_once(dirname(__FILE__) . '/RSAKey.php'); // helper class for mange single key
require_once(dirname(__FILE__) . '/RSAKeyPair.php'); // helper class for manage key pair

/**
 * Crypt_RSA class.
 *
 * Provides key generation, encryption, decryption, signing and sign validating methods.
 * See comments before each method to understand its purpose.
 *
 * Example usage:
 *     // 1024-bit key pair generation
 *     $key_pair = Crypt_RSA::generateKeyPair('BigInt', 1024);
 *
 *     // creating Crypt_RSA object
 *     $rsa_obj = new Crypt_RSA;
 *
 *     // encryption (usually using public key)
 *     $enc_data = $rsa_obj->encrypt($plain_data, $key_pair->getPublicKey());
 *
 *     // decryption (usually using private key)
 *     $plain_data = $rsa_obj->decrypt($enc_data, $key_pair->getPrivateKey());
 *
 *     // signing
 *     $signature = $rsa_obj->createSign($document, $key_pair->getPrivateKey());
 *
 *     // signature checking
 *     $is_valid = $rsa_obj->validateSign($document, $signature, $key_pair->getPublicKey());
 *
 *     // signing many documents
 *     $rsa_obj = new Crypt_RSA(array('private_key' => $private_key));
 *     $sign_1 = $rsa_obj->sign($doc_1);
 *     $sign_2 = $rsa_obj->sign($doc_2);
 *     //...
 *     $sign_n = $rsa_obj->sign($doc_n);
 *
 *     // changing default hash function, which is using for sign
 *     // creating/validation
 *     $rsa_obj->setParams(array('hash_func' => 'md5'));
 *
 * @category   Encryption
 * @package    Crypt_RSA
 * @author     Alexander Valyalkin <valyala@gmail.com>
 * @copyright  2005 Alexander Valyalkin
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @link       http://pear.php.net/package/Crypt_Blowfish
 * @version    @package_version@
 * @access     public
 */
class Crypt_RSA
{
    /*
     * Instance of math library wrapper class, which is used to
     * manipulate large integers in RSA algorithm.
     * See MathClasses/RSAMathBigInt.php for details.
     *
     * @var string
     * @access private
     */
    var $_math_obj;

    /*
     * key for encryption, which is used by encrypt() method
     *
     * @var Crypt_RSAKey
     * @access private
     */
    var $_enc_key;

    /*
     * key for decryption, which is used by decrypt() method
     *
     * @var Crypt_RSAKey
     * @access private
     */
    var $_dec_key;

    /*
     * public key, which is used by validateSign() method
     *
     * @var Crypt_RSAKey
     * @access private
     */
    var $_public_key;

    /*
     * private key, which is used by createSign() method
     *
     * @var Crypt_RSAKey
     * @access private
     */
    var $_private_key;

    /*
     * name of hash function, which is used by validateSign()
     * and createSign() methods. Default hash function is SHA-1
     *
     * @var string
     * @access private
     */
    var $_hash_func = 'sha1';

    /**
     * Crypt_RSA constructor.
     * Accepts name of math lib $math_lib_name, which is used to handle
     * math operations with large integers in RSA algorithm.
     * Each math lib must be wrapped by class with interface,
     * identical to Crypt_RSAMathBigInt class.
     * See MathClasses/RSAMathBigInt.php for details.
     *
     * Math wrapper classes must have the following names:
     *     Crypt_RSAMath{math_lib_name}
     * where {math_lib_name} - name of math library.
     * These classes must be placed in MathClasses folder in
     * file with name
     *     RSAMath{math_lib_name}.php
     *
     * Accepts optional associative array of parameters, such as:
     * enc_key, dec_key, private_key, public_key, hash_func.
     * See setParams() method for more detail description of these parameters
     *
     * @param string $math_lib_name
     * @param array $params
     * @access public
     */
    function Crypt_RSA($math_lib_name, $params = null)
    {
        // try to load wrapper class for math library $math_lib
        $this->_math_obj = Crypt_RSAMathLoader::loadLib($math_lib_name);

        if (!is_null($params)) {
            $this->setParams($params);
        }
    }

    /**
     * Validates $key
     * This function can be called statically:
     *    $is_valid = Crypt_RSA::isValidKey($key)
     *
     * Returns true, if $key is valid Crypt_RSA key, else returns false
     *
     * @param Crypt_RSAKey $key
     * @return bool
     * @access public
     */
    function isValidKey($key)
    {
        return (is_object($key) && strtolower(get_class($key)) == 'crypt_rsakey');
    }

    /**
     * Accepts any combination of available parameters as associative array:
     *     enc_key - encryption key for encrypt() method
     *     dec_key - decryption key for decrypt() method
     *     public_key - key for validateSign() method
     *     private_key - key for createSign() method
     *     hash_func - name of hash function, which will be used to create and validate sign
     *
     * @param array $params
     * @access public
     */
    function setParams($params)
    {
        if (isset($params['enc_key'])) {
            if (Crypt_RSA::isValidKey($params['enc_key'])) {
                $this->_enc_key = $params['enc_key'];
            } else {
                exit('Crypt_RSA::setParams(): wrong [enc_key]. It must be an object of Crypt_RSAKey class');
            }
        }
        if (isset($params['dec_key'])) {
            if (Crypt_RSA::isValidKey($params['dec_key'])) {
                $this->_dec_key = $params['dec_key'];
            } else {
                exit('Crypt_RSA::setParams(): wrong [dec_key]. It must be an object of Crypt_RSAKey class');
            }
        }
        if (isset($params['private_key'])) {
            if (Crypt_RSA::isValidKey($params['private_key'])) {
                if ($params['private_key']->getKeyType() != 'private') {
                    exit('Crypt_RSA::setParams(): [private_key] must be private');
                }
                $this->_private_key = $params['private_key'];
            } else {
                exit('Crypt_RSA::setParams(): wrong [private_key]. It must be an object of Crypt_RSAKey class');
            }
        }
        if (isset($params['public_key'])) {
            if (Crypt_RSA::isValidKey($params['public_key'])) {
                if ($params['public_key']->getKeyType() != 'public') {
                    exit('Crypt_RSA::setParams(): [public_key] must be public');
                }
                $this->_public_key = $params['public_key'];
            } else {
                exit('Crypt_RSA::setParams(): wrong [public_key]. It must be an object of Crypt_RSAKey class');
            }
        }
        if (isset($params['hash_func'])) {
            if (!function_exists($params['hash_func'])) {
                exit('cannot find hash function with name [' . $params['hash_func'] . ']');
            }
            $this->_hash_func = $params['hash_func'];
        }
    }

    /**
     * Ecnrypts $plain_data by $this->_enc_key or $key.
     *
     * Returns encrypted data.
     *
     * @param string $plain_data
     * @param Crypt_RSAKey $key
     * @return string
     * @access public
     */
    function encrypt($plain_data, $key = null)
    {
        if (is_null($key)) {
            // use current encryption key
            $key = $this->_enc_key;
        } elseif (!Crypt_RSA::isValidKey($key)) {
            exit('Crypt_RSA::encrypt(): invalid key. It must be an object of Crypt_RSAKey class');
        }

        $plain_data = $this->_math_obj->bin2int($plain_data);
        $exp = $this->_math_obj->bin2int($key->getExponent());
        $modulus = $this->_math_obj->bin2int($key->getModulus());

        // divide into chunks
        $data_len = $this->_math_obj->bitLen($plain_data);
        $chunk_len = $key->getKeyLength() - 1;
        $block_len = (int) ceil($chunk_len / 8);
        $curr_pos = 0;
        $enc_data = '';
        while ($curr_pos < $data_len) {
            $tmp = $this->_math_obj->subint($plain_data, $curr_pos, $chunk_len);
            $enc_data .= str_pad(
                $this->_math_obj->int2bin($this->_math_obj->powmod($tmp, $exp, $modulus)),
                $block_len,
                "\0"
            );
            $curr_pos += $chunk_len;
        }
        return base64_encode($enc_data);
    }

    /**
     * Decrypts $enc_data by $this->_dec_key or $key.
     *
     * Returns decrypted plaintext
     *
     * @param string $enc_data
     * @param Crypt_RSAKey $key
     * @return string
     * @access public
     */
    function decrypt($enc_data, $key = null)
    {
        if (is_null($key)) {
            // use current decryption key
            $key = $this->_dec_key;
        } elseif (!Crypt_RSA::isValidKey($key)) {
            exit('Crypt_RSA::decrypt(): invalid key. It must be an object of Crypt_RSAKey class');
        }

        $enc_data = base64_decode($enc_data);
        $exp = $this->_math_obj->bin2int($key->getExponent());
        $modulus = $this->_math_obj->bin2int($key->getModulus());

        $data_len = strlen($enc_data);
        $chunk_len = $key->getKeyLength() - 1;
        $block_len = (int) ceil($chunk_len / 8);
        $curr_pos = 0;
        $plain_data = $this->_math_obj->bin2int("\0");
        while ($curr_pos < $data_len) {
            $tmp = $this->_math_obj->bin2int(substr($enc_data, $curr_pos, $block_len));
            $tmp = $this->_math_obj->powmod($tmp, $exp, $modulus);
            $plain_data = $this->_math_obj->bitOr($plain_data, $tmp, $curr_pos);
            $curr_pos += $block_len;
        }

        return $this->_math_obj->int2bin($plain_data);
    }

    /**
     * Creates sign for document $document, using $this->_private_key of $private_key
     * as Crypt_RSA key and $this->_hash_func of $hash_func as hash function.
     *
     * Returns created sign.
     *
     * @param string $document
     * @param Crypt_RSAKey $private_key
     * @param string $hash_func
     * @return string
     * @access public
     */
    function createSign($document, $private_key = null, $hash_func = null)
    {
        // check private key
        if (is_null($private_key)) {
            $private_key = $this->_private_key;
        } elseif (!Crypt_RSA::isValidKey($private_key)) {
            exit('Crypt_RSA::createSign(): invalid private key. It must be an object of Crypt_RSAKey class');
        }
        if ($private_key->getKeyType() != 'private') {
            exit('Crypt_RSA::createSign(): signing key must be private');
        }

        // check hash_func
        if (is_null($hash_func)) {
            $hash_func = $this->_hash_func;
        }
        if (!function_exists($hash_func)) {
            exit('cannot find hash function with name [' . $hash_func . ']');
        }

        return $this->encrypt($hash_func($document), $private_key);
    }

    /**
     * Validates $signature for document $document with public key $this->_public_key
     * of $public_key and hash function $this->_hash_func of $hash_func.
     *
     * Returns true, if $signature is valid for $document, else returns false.
     *
     * @param string $document
     * @param string $signature
     * @param Crypt_RSAKey $public_key
     * @param string $hash_func
     * @return bool
     * @access public
     */
    function validateSign($document, $signature, $public_key = null, $hash_func = null)
    {
        // check public key
        if (is_null($public_key)) {
            $public_key = $this->_public_key;
        } elseif (!Crypt_RSA::isValidKey($public_key)) {
            exit('Crypt_RSA::validateSign(): invalid public key. It must be an object of Crypt_RSAKey class');
        }
        if ($public_key->getKeyType() != 'public') {
            exit('Crypt_RSA::validateSign(): validating key must be public');
        }

        // check hash_func
        if (is_null($hash_func)) {
            $hash_func = $this->_hash_func;
        }
        if (!function_exists($hash_func)) {
            exit('cannot find hash function with name [' . $hash_func . ']');
        }

        return $hash_func($document) == $this->decrypt($signature, $public_key);
    }

    /**
     * Generates new key pair with length $bit_len using default random generator
     * or $random_generator if it passed.
     *
     * @param int $bit_len
     * @param string $random_generator
     * @return Crypt_RSAKeyPair
     * @access public
     */
    function generateKeyPair($bit_len, $random_generator = null)
    {
        return new Crypt_RSAKeyPair(
            Crypt_RSAMathLoader::getLibNameByObj($this->_math_obj),
            $bit_len,
            $random_generator
        );
    }
}

?>