<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

/**
 * Crypt_RSA allows to do following operations:
 *     - key pair generation
 *     - encryption and decryption
 *     - signing and sign validation
 *
 * PHP versions 4 and 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   Encryption
 * @package    Crypt_RSA
 * @author     Alexander Valyalkin <valyala@gmail.com>
 * @copyright  2005 Alexander Valyalkin
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    1.0.0
 * @link       http://pear.php.net/package/Crypt_RSA
 */

require_once(dirname(__FILE__) . '/RSAMathLoader.php'); // loader for math wrappers

/**
 * Crypt_RSAKey class.
 *
 * Provides methods to retrieve key length, modulus and exponent parts of key
 * as strings or as big_int numbers, to save key into string or load it from string.
 * See comments before each method to understand its purpose.
 *
 * Example usage:
 *    // create new 1024-bit key pair
 *    $key_pair = new Crypt_RSAKeyPair(1024);
 * 
 *    // get public key (its class is Crypt_RSAKey)
 *    $key = $key_pair->getPublicKey();
 *
 *    // get key length
 *    $len = $key->getKeyLength();
 *
 *    // get modulus as string
 *    $modulus = $key->getModulus();
 *
 *    // get exponent as string
 *    $exponent = $key->getExponent();
 *
 *    // get string represenation of key (use it instead of serialize)
 *    $key_in_str = $key->toString();
 *
 *    // restore key object from string using 'BigInt' math library
 *    $key = Crypt_RSAKey::fromString($key_in_str, 'BigInt');
 *
 * @category   Encryption
 * @package    Crypt_RSA
 * @author     Alexander Valyalkin <valyala@gmail.com>
 * @copyright  2005 Alexander Valyalkin
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @link       http://pear.php.net/package/Crypt_Blowfish
 * @version    @package_version@
 * @access     public
 */
class Crypt_RSAKey
{
    /*
     * Instance of math library wrapper class, which is used to
     * manipulate large integers in RSA algorithm.
     * See MathClasses/RSAMathBigInt.php for details.
     *
     * @var string
     * @access private
     */
    var $_math_obj;

    /*
     * shared modulus
     *
     * @var string
     * @access private
     */
    var $_modulus;

    /*
     * exponent
     *
     * @var string
     * @access private
     */
    var $_exp;

    /*
     * key type (private or public)
     *
     * @var string
     * @access private
     */
    var $_key_type;

    /**
     * Crypt_RSAKey constructor.
     * Accepts name of math lib $math_lib_name, which is used to handle
     * math operations with large integers in RSA algorithm.
     * Each math lib must be wrapped by class with interface,
     * identical to Crypt_RSAMathBigInt class.
     * See MathClasses/RSAMathBigInt.php for details.
     *
     * Math wrapper classes must have the following names:
     *     Crypt_RSAMath{math_lib_name}
     * where {math_lib_name} - name of math library.
     * These classes must be placed in MathClasses folder in
     * file with name
     *     RSAMath{math_lib_name}.php
     *
     * Accepts modulus, exponent and key type (public, private) as parameters,
     * which goes after $math_lib_name
     *
     * @param string $modulus
     * @param string $exp
     * @param string $key_type
     * @access public
     */
    function Crypt_RSAKey($math_lib_name, $modulus, $exp, $key_type)
    {
        // try to load wrapper class for math library $math_lib
        $this->_math_obj = Crypt_RSAMathLoader::loadLib($math_lib_name);

        $this->_modulus = $modulus;
        $this->_exp = $exp;

        if (!in_array($key_type, array('private', 'public'))) {
            exit('Crypt_RSAKey::Crypt_RSAKey(): $key_type must be private or public');
        }
        $this->_key_type = $key_type;

        /* check length of modulus & exponent ( abs(modulus) > abs(exp) ) */
        $mod_num = $this->_math_obj->bin2int($this->_modulus);
        $exp_num = $this->_math_obj->bin2int($this->_exp);
        if ($this->_math_obj->cmpAbs($mod_num, $exp_num) <= 0) {
            exit('Crypt_RSAKey::Crypt_RSAkey(): $modulus must be greater than $exponent');
        }
    }

    /**
     * Returns bit length of the key
     *
     * @return int
     * @access public
     */
    function getKeyLength()
    {
        return $this->_math_obj->bitLen($this->_math_obj->bin2int($this->_modulus));
    }

    /**
     * Returns modulus part of the key as string,
     * which can be used to construct new Crypt_RSAKey object.
     *
     * @return string
     * @access public
     */
    function getModulus()
    {
        return $this->_modulus;
    }

    /**
     * Returns exponent part of the key as string,
     * which can be used to construct new Crypt_RSAKey object.
     *
     * @return string
     * @access public
     */
    function getExponent()
    {
        return $this->_exp;
    }

    /**
     * Returns key type (public, private)
     *
     * @return string
     * @access public
     */
    function getKeyType()
    {
        return $this->_key_type;
    }

    /**
     * Returns string representation of key
     *
     * @return string
     * @access public
     */
    function toString()
    {
        return base64_encode(
            serialize(array(
                $this->_modulus,
                $this->_exp,
                $this->_key_type,
            ))
        );
    }

    /**
     * Returns Crypt_RSAKey object, recovered from
     * string representation of key.
     * $math_lib_name - is the name of math library,
     * which will be used to manipulate this object.
     *
     * This function can be called statically:
     *     $key = Crypt_RSAKey::fromString($key_in_string, 'BigInt');
     * If function is called statically, we need to pass name
     * of the math library to function.
     *
     * @return Crypt_RSAKey
     * @access public
     */
    function fromString($str, $math_lib_name = null)
    {
        if (is_null($math_lib_name)) {
            if (is_null($this)) {
                exit('Crypt_RSAKey::fromString(): missing $math_lib_name as second parameter during static call');
            }
            $math_lib_name = Crypt_RSAMathLoader::getLibNameByObj($this->_math_obj);
        }

        list($modulus, $exponent, $key_type) = unserialize(base64_decode($str));
        return new Crypt_RSAKey($math_lib_name, $modulus, $exponent, $key_type);
    }
}

?>