<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

/**
 * Crypt_RSA allows to do following operations:
 *     - key pair generation
 *     - encryption and decryption
 *     - signing and sign validation
 *
 * PHP versions 4 and 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   Encryption
 * @package    Crypt_RSA
 * @author     Alexander Valyalkin <valyala@gmail.com>
 * @copyright  2005 Alexander Valyalkin
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    1.0.0
 * @link       http://pear.php.net/package/Crypt_RSA
 */

require_once(dirname(__FILE__) . '/RSAMathLoader.php'); // loader for math wrappers
require_once(dirname(__FILE__) . '/RSAKey.php'); // helper class for manage single key

/**
 * Crypt_RSAKeyPair class.
 *
 * Provides key generation, public and private key retrieving methods.
 * See comments before each method to understand its purpose.
 *
 * Example usage:
 *    // create new 1024-bit key pair
 *    $key_pair = new Crypt_RSAKeyPair('BigInt', 1024);
 * 
 *    // get public key
 *    $public_key = $key_pair->getPublicKey();
 * 
 *    // get private key
 *    $private_key = $key_pair->getPrivateKey();
 * 
 *    // generate new 512-bit key pair
 *    $key_pair->generate(512);
 *
 *    // get key pair length
 *    $length = $key_pair->getKeyLength();
 *
 *    // set random generator to $func_name, where $func_name
 *    // consists name of random generator function. See comments
 *    // befor setRandomGenerator() method for more info.
 *    $key_pair->setRandomGenerator($func_name);
 *
 * @category   Encryption
 * @package    Crypt_RSA
 * @author     Alexander Valyalkin <valyala@gmail.com>
 * @copyright  2005 Alexander Valyalkin
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @link       http://pear.php.net/package/Crypt_RSA
 * @version    @package_version@
 * @access     public
 */
class Crypt_RSAKeyPair
{
    /*
     * Instance of math library wrapper class, which is used to
     * manipulate large integers in RSA algorithm.
     * See MathClasses/RSAMathBigInt.php for details.
     *
     * @var string
     * @access private
     */
    var $_math_obj;

    /*
     * length of each key in the key pair
     *
     * @var int
     * @access private
     */
    var $_key_len;

    /*
     * public key
     *
     * @var Crypt_RSAKey
     * @access private
     */
    var $_public_key;

    /*
     * private key
     *
     * @var Crypt_RSAKey
     * @access private
     */
    var $_private_key;

    /*
     * name of function, which is used as random generator
     *
     * @var string
     * @access private
     */
    var $_random_generator;

    /**
     * Crypt_RSAKeyPair constructor.
     * Accepts name of math lib $math_lib_name, which is used to handle
     * math operations with large integers in RSA algorithm.
     * Each math lib must be wrapped by class with interface,
     * identical to Crypt_RSAMathBigInt class.
     * See MathClasses/RSAMathBigInt.php for details.
     *
     * Math wrapper classes must have the following names:
     *     Crypt_RSAMath{math_lib_name}
     * where {math_lib_name} - name of math library.
     * These classes must be placed in MathClasses folder in
     * file with name
     *     RSAMath{math_lib_name}.php
     *
     * Accepts length of each key in key pair $key_len and
     * $random_generator function name, which will be used as
     * random generator as 2nd and 3rd parameters.
     * $random_genrator is optional parameter
     *
     * Crypt_RSAKeyPair constructor generates new key pair.
     *
     * @param int $key_len
     * @param string $random_generator
     * @access public
     */
    function Crypt_RSAKeyPair($math_lib_name, $key_len, $random_generator = null)
    {
        // try to load wrapper class for math library $math_lib
        $this->_math_obj = Crypt_RSAMathLoader::loadLib($math_lib_name);

        $this->_key_len = null;

        $this->setRandomGenerator($random_generator);
        $this->generate($key_len);
    }

    /**
     * Generates new Crypt_RSA key pair with length $key_len.
     * If $key_len is missed, use an old key length from $this->getKeyLength()
     *
     * @param int $key_len
     * @access public
     */
    function generate($key_len = null)
    {
        if (is_null($key_len)) {
            // use an old key length
            $key_len = $this->getKeyLength();
            if (is_null($key_len)) {
                exit('Crypt_RSAKeyPair::generate(): missing $key_len parameter');
            }
        }
        // align $key_len to 8 bits
        if ($key_len & 7) {
            $key_len += 8 - ($key_len % 8);
        }
        // store key length in the object
        $this->_key_len = $key_len;

        // generate two primes p and q
        $p_len = (int) ($key_len / 2) + 1;
        $q_len = $key_len - $p_len;
        $p = $this->_math_obj->getRand($p_len, $this->_random_generator, true);
        $p = $this->_math_obj->nextPrime($p);
        do {
            do {
                $q = $this->_math_obj->getRand($q_len, $this->_random_generator, true);
                $tmp_len = $this->_math_obj->bitLen($this->_math_obj->mul($p, $q));
                if ($tmp_len < $key_len) $q_len++;
                elseif ($tmp_len > $key_len) $q_len--;
            } while ($tmp_len != $key_len);
            $q = $this->_math_obj->nextPrime($q);
            $tmp = $this->_math_obj->mul($p, $q);
        } while ($this->_math_obj->bitLen($tmp) != $key_len);
        // $n - is shared modulus
        $n = $this->_math_obj->mul($p, $q);
        // generate public ($e) and private ($d) keys
        $pq = $this->_math_obj->mul($this->_math_obj->dec($p), $this->_math_obj->dec($q));
        do {
            $e = $this->_math_obj->getRand($q_len, $this->_random_generator);
        } while (!$this->_math_obj->isZero($this->_math_obj->dec($this->_math_obj->gcd($e, $pq))));
        $d = $this->_math_obj->invmod($e, $pq);

        $modulus = $this->_math_obj->int2bin($n);
        $public_exp = $this->_math_obj->int2bin($e);
        $private_exp = $this->_math_obj->int2bin($d);

        $this->_public_key = new Crypt_RSAKey(
            Crypt_RSAMathLoader::getLibNameByObj($this->_math_obj),
            $modulus,
            $public_exp,
            'public'
        );
        $this->_private_key = new Crypt_RSAKey(
            Crypt_RSAMathLoader::getLibNameByObj($this->_math_obj),
            $modulus,
            $private_exp,
            'private'
        );
    }

    /**
     * Returns public key from the pair
     *
     * @return Crypt_RSAKey
     * @access public
     */
    function getPublicKey()
    {
        return $this->_public_key;
    }

    /**
     * Returns private key from the pair
     *
     * @return Crypt_RSAKey
     * @access public
     */
    function getPrivateKey()
    {
        return $this->_private_key;
    }

    /**
     * Sets name of random generator function for key generation.
     * If parameter is skipped, then sets to default random generator.
     *
     * Random generator function must return integer with at least 8 lower
     * significant bits, which will be used as random values.
     *
     * @param string $random_generator
     * @access public
     */
    function setRandomGenerator($random_generator = null)
    {
        static $default_random_generator = null;

        if (is_string($random_generator)) {
            // set user's random generator
            if (!function_exists($random_generator)) {
                exit("Crypt_RSAKeyPair::setRandomGenerator(): can't find random generator function with name [{$random_generator}]");
            }
            $this->_random_generator = $random_generator;
        } else {
            // set default random generator
            $this->_random_generator = is_null($default_random_generator) ?
                ($default_random_generator = create_function('', '$a=explode(" ",microtime());return(int)($a[0]*1000000);')) :
                $default_random_generator;
        }
    }

    /**
     * Returns length of each key in the key pair
     *
     * @return int
     * @access public
     */
    function getKeyLength()
    {
        return $this->_key_len;
    }
}

?>