<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

/**
 * Crypt_RSA allows to do following operations:
 *     - key pair generation
 *     - encryption and decryption
 *     - signing and sign validation
 *
 * PHP versions 4 and 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   Encryption
 * @package    Crypt_RSA
 * @author     Alexander Valyalkin <valyala@gmail.com>
 * @copyright  2005 Alexander Valyalkin
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    1.0.0
 * @link       http://pear.php.net/package/Crypt_RSA
 */

/**
 * Crypt_RSAMathLoader class.
 *
 * Provides loadLib static function to load RSA wrapper class for math library
 *
 * Example usage:
 *    // load BigInt wrapper
 *    $big_int_wrapper = Crypt_RSAMathLoader::loadLib('BigInt');
 * 
 *    // load BCMath wrapper
 *    $bcmath_wrapper = Crypt_RSAMathLoader::loadLib('BCMath');
 * 
 *    // load GMP wrapper
 *    $gmp_wrapper = Crypt_RSAMathLoader::loadLib('GMP');
 *
 * @category   Encryption
 * @package    Crypt_RSA
 * @author     Alexander Valyalkin <valyala@gmail.com>
 * @copyright  2005 Alexander Valyalkin
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @link       http://pear.php.net/package/Crypt_RSA
 * @version    @package_version@
 * @access     public
 */
class Crypt_RSAMathLoader
{
    /**
     * Loads wrapper class for math library $lib_name.
     * Wrapper classes are placed in MathClasses folder.
     *
     * This is a static function:
     *    // load BigInt wrapper
     *    $big_int_wrapper = Crypt_RSAMathLoader::loadLib('BigInt');
     *
     * @param string $lib_name
     * @return object of wrapper class for math library $lib_name
     * @access public
     */
    function loadLib($lib_name)
    {
        $class_name = 'Crypt_RSAMath' . $lib_name;
        $class_filename = dirname(__FILE__) . '/MathClasses/RSAMath' . $lib_name . '.php';

        if (!is_file($class_filename)) {
            exit("can't find file [{$class_filename}] for math library [{$lib_name}]");
        }
        require_once($class_filename);
        if (!class_exists($class_name)) {
            exit("can't find class [{$class_name}] in file [{$class_filename}]");
        }

        // create and return new object of class $class_name
        return new $class_name;
    }

    /**
     * Returns library name for instance of wrapper class for that math library.
     *
     * @param object of wrapper class for math library $lib_name
     * @return string
     * @access public
     */
    function getLibNameByObj($math_obj)
    {
        if (!preg_match('/^Crypt_RSAMath(.*)$/i', get_class($math_obj), $matches)) {
            exit('Crypt_RSAMathLoader::getLibNameByObj(): incorrect class of object $math_obj');
        }
        return $matches[1];
    }
}

?>